<?php

require 'assets/php/models/TimeSlot.php';

$db_path = $_SERVER['DOCUMENT_ROOT'] . '/assets/database.sqlite';

$query_result;
$wanted_slots;

try {
  /*Setup SQL Interface*/
  $pdo = new PDO('sqlite:' . $db_path);
  $pdo->setAttribute(PDO::ATTR_DEFAULT_FETCH_MODE, PDO::FETCH_ASSOC);
  $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

  /*Prepare query*/
  $query = $pdo->prepare("SELECT id, type, start_time, end_time, location
    FROM schedule NATURAL JOIN rooms
    WHERE type LIKE :type AND date(date) = date(:date)
    AND time(end_time) > time(:start_time) AND time(start_time) < time(:end_time)");

  /**
   * QUERY DESCRIPTION : 
   * Select all slots mathcing the date & room and where the time slot
   * is overllaping with the requested one. 
   */

  /*Execute query*/
  $query->execute([
    'type' => $_GET['room_type'],
    'date' => $_GET['date'],
    'start_time' => $_GET['start_time'],
    'end_time' => $_GET['end_time']
  ]);
  $query_result = $query->fetchAll(PDO::FETCH_CLASS, 'TimeSlot');

  /* Prepare wanted times slots in each room*/
  $wanted_slots =
    array_map(
      /*Create new wanted slots for each unique room*/
      function (Room $r): TimeSlot {
        $slot = new TimeSlot();
        $slot->start_time = $_GET['start_time'];
        $slot->end_time = $_GET['end_time'];
        $slot->room = clone $r;
        return $slot;
      },
      /*Construct an array of all unique rooms*/
      array_unique(array_map(
        function (TimeSlot $x): Room {
          return $x->room;
        },
        $query_result
      ), SORT_REGULAR)
    );
} catch (PDOException $exception) {
  echo var_dump($exception->getMessage());
}

?>
<!DOCTYPE html>
<html lang="en">

<head>
  <title>UFHOUR</title>

  <!-- Head -->
  <?php require("assets/php/head.php"); ?>
  <link rel="stylesheet" href="assets/css/search.css">
</head>

<body>
  <main>
    <header>
      <h1>UFHOUR</h1>
      <p>Recherche de salles libres de l'UFR Math-Info</p>
    </header>
    <h2>Résultats de la recherche</h2>
    <hr>
    <section>

      <?php
      foreach ($query_result as $overlap) {
        $overlap->display();
      }

      echo "Scis";

      $final_results = [];
      foreach ($wanted_slots as $span) {
        $scissors = array_filter(
          $query_result,
          function (TimeSlot $e) use ($span): bool {
            return $e->room->id === $span->room->id;
          }
        );

        $cutouts = [$span];
        foreach ($scissors as $overlap) {
          $temp = [];
          foreach ($cutouts as $x)
            $temp = array_merge($temp, $x->cut($overlap));
          $cutouts = $temp;
        }
        $final_results = array_merge($final_results, $cutouts);
      }


      foreach ($final_results as $slot)
        $slot->display();


      echo "test";
      $test_room = new Room();
      $test_room->id = "UWU";
      $test_room->location = 10;
      $test_room->type = "tp";

      $test_span = new TimeSlot();
      $test_span->start_time = "09:00";
      $test_span->end_time = "15:00";
      $test_span->room = $test_room;

      $test_cut = new TimeSlot();
      $test_cut->start_time = "13:00";
      $test_cut->end_time = "15:00";
      $test_cut->room = $test_room;


      $res = $test_span->cut($test_cut);
      foreach ($res as $x) {
        $x->display();
      }
      ?>




    </section>
  </main>
  <?php include "assets/php/cookies.php"; ?>
  <?php include "assets/php/nav.php"; ?>
</body>

</html>